# encoding: utf-8

from hashlib import md5
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from ideabox.policy import _
from ideabox.policy.faceted.subscriber import Randomizer
from plone import api
from plone.app.z3cform.widget import RelatedItemsFieldWidget
from plone.autoform import directives
from plone.supermodel.model import Schema
from plone.tiles import Tile
from random import shuffle
from zope import schema

import time


class IPriorityActionTile(Schema):
    """A tile that displays a listing of content items"""

    title = schema.TextLine(
        title=_(u"Title"), required=True, default=_(u"Priority action")
    )

    uid = schema.Choice(
        title=_(u"Select a container"),
        required=True,
        vocabulary="plone.app.vocabularies.Catalog",
    )
    directives.widget(
        "uid",
        RelatedItemsFieldWidget,
        pattern_options={"selectableTypes": ["campaign", "Folder"]},
    )


class TileRandomizer(Randomizer):
    def __init__(self):
        self.key = md5(str(time.time() // 60 * 5).encode("utf-8")).hexdigest()


class BasePriorityActionTile(Tile):
    template = ViewPageTemplateFile("templates/priority_action.pt")

    def __call__(self):
        return self.template()

    @property
    def title(self):
        return self.data.get("title") or _(u"Priority action")

    @property
    def default_image(self):
        return "{0}/project_default.jpg".format(api.portal.get().absolute_url())

    def get_contents(self, container):
        raise NotImplementedError

    def contents(self):
        uid = self.data["uid"]
        data = {"url": "", "results": []}
        if uid:
            container = api.content.get(UID=uid)
            if container:
                data["url"] = container.absolute_url()
                data["results"] = self.get_contents(container)
        return data


class LatestPriorityActionTile(BasePriorityActionTile):
    def get_contents(self, container):
        return api.content.find(
            context=container,
            portal_type="priority_action",
            sort_on="created",
            sort_order="reverse",
        )[:6]

    def folder_projects(self):
        folder = api.content.find(portal_type="Folder", id="projets")
        if len(folder) == 1:
            return folder[0].getURL()
        return False


class RandomPriorityActionTile(BasePriorityActionTile):
    def get_contents(self, container):
        query = {}
        randomizer = TileRandomizer()
        query["sort_on"] = randomizer.random_sort_key
        query["sort_order"] = randomizer.random_sort_order
        result = api.content.find(
            context=container, portal_type="priority_action", **query
        )[:200]
        shuffle(result)
        return result[:6]



# -*- coding: utf-8 -*-

from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from ideabox.policy import _
from plone import api
from plone.app.users.browser.register import RegistrationForm
from plone.app.users.utils import notifyWidgetActionExecutionError
from z3c.form.field import Fields
from zope import schema
from zope.component import adapter
from zope.interface import Interface
from zope.interface import implementer


class ICustomRegisterSchema(Interface):

    legal_conditions = schema.Bool(
        title=_(u"I Accept Legal terms and conditions"), required=True
    )


@adapter(Interface)
@implementer(ICustomRegisterSchema)
class CustomRegistrationAdapter(object):
    def __init__(self, context):
        self.context = context


class CustomRegistrationForm(RegistrationForm):
    template = ViewPageTemplateFile("templates/register_form.pt")

    def updateFields(self):
        super(CustomRegistrationForm, self).updateFields()
        self.fields += Fields(ICustomRegisterSchema)
        portal_url = api.portal.get().absolute_url()
        gpdr_url = "/".join([portal_url, "gdpr-view"])
        msgid = _(
            u"see_legal_conditions",
            default=u'See <a href="${legal_conditions_url}">legal terms and conditions</a>.',
            mapping={u"legal_conditions_url": gpdr_url},
        )
        legal_description = self.context.translate(msgid)
        self.fields["legal_conditions"].field.description = legal_description

    def validate_registration(self, action, data):
        super(CustomRegistrationForm, self).validate_registration(action, data)
        if not data.get("legal_conditions"):
            err_str = _(u"You need to accept our legal terms and conditions.")
            notifyWidgetActionExecutionError(action, "legal_conditions", err_str)



# -*- coding: utf-8 -*-

from Products.CMFPlone.interfaces import INonInstallable
from collective.taxonomy.exportimport import TaxonomyImportExportAdapter
from collective.taxonomy.factory import registerTaxonomy
from collective.taxonomy.interfaces import ITaxonomy
from eea.facetednavigation.layout.layout import FacetedLayout
from ideabox.policy import _
from ideabox.policy import utils
from plone import api
from plone.dexterity.interfaces import IDexterityFTI
from zope.component import queryUtility
from zope.i18n import translate
from zope.i18n.interfaces import ITranslationDomain
from zope.interface import implementer
from zope.schema.interfaces import IVocabularyFactory

import os


@implementer(INonInstallable)
class HiddenProfiles(object):
    def getNonInstallableProfiles(self):
        """Hide uninstall profile from site-creation and quickinstaller."""
        return ["ideabox.policy:uninstall"]


def post_install(context):
    """Post install script"""
    add_taxonomies()
    portal = api.portal.get()
    if portal.get("projets") is None:
        project = api.content.create(
            type="Folder", id="projets", title="Projets", container=portal
        )
        api.content.transition(obj=project, transition="publish")
        add_behavior(
            "Folder",
            "eea.facetednavigation.subtypes.interfaces.IPossibleFacetedNavigable",
        )
        _activate_faceted_navigation(project, True, "/faceted/config/projets.xml")
        project_layout = FacetedLayout(project)
        project_layout.update_layout(layout="faceted-project")
        utils.disable_portlets(project)
        _publish(project)
    if portal.get("participer") is None:
        participate = api.content.create(
            type="Folder", id="participer", title="Participer", container=portal
        )
        utils.disable_portlets(participate)
        _publish(participate)
    if portal.get("plus-dinfos") is None:
        infos = api.content.create(
            type="Folder", id="plus-dinfos", title=u"Plus d'infos", container=portal
        )
        utils.disable_portlets(infos, disabled=("plone.rightcolumn",))
        _publish(infos)

    add_behavior(
        "Collection",
        "eea.facetednavigation.subtypes.interfaces.IPossibleFacetedNavigable",
    )
    if "news" in portal:
        _activate_faceted_navigation(
            portal["news"]["aggregator"], True, "/faceted/config/news.xml"
        )
        utils.disable_portlets(portal["news"])
        news_layout = FacetedLayout(portal["news"]["aggregator"])
        news_layout.update_layout(layout="faceted-news")

    allowed_sizes = api.portal.get_registry_record("plone.allowed_sizes")
    scales = ("banner 1920:800", "project_faceted 450:300", "evenement 300:300")
    for scale in scales:
        if scale not in allowed_sizes:
            allowed_sizes.append(scale)
    api.portal.set_registry_record("plone.allowed_sizes", allowed_sizes)

    api.portal.set_registry_record(
        "collective.behavior.banner.browser.controlpanel.IBannerSettingsSchema.banner_scale",
        "banner",
    )


def add_taxonomies():
    # TODO fix in test... and also in collective.taxonomy.
    current_lang = api.portal.get_current_language()[:2]

    data_theme = {
        "taxonomy": "theme",
        "field_title": translate(_("Theme"), target_language=current_lang),
        "field_description": "",
        "default_language": "fr",
        "filename": "taxonomy-settings-theme.xml",
    }

    data_district = {
        "taxonomy": "district",
        "field_title": translate(_("District (Project)"), target_language=current_lang),
        "field_description": "",
        "default_language": "fr",
        "filename": "taxonomy-settings-district.xml",
    }

    data_iam = {
        "taxonomy": "iam",
        "field_title": translate(_("I am"), target_language=current_lang),
        "field_description": "",
        "default_language": "fr",
        "filename": "taxonomy-settings-iam.xml",
    }

    data_locality = {
        "taxonomy": "locality",
        "field_title": translate(
            _("Locality (Registration form)"), target_language=current_lang
        ),
        "field_description": "",
        "default_language": "fr",
        "filename": "taxonomy-settings-locality.xml",
    }

    portal = api.portal.get()
    sm = portal.getSiteManager()
    theme_item = "collective.taxonomy.theme"
    district_item = "collective.taxonomy.district"
    iam_item = "collective.taxonomy.iam"
    locality_item = "collective.taxonomy.locality"
    utility_theme = sm.queryUtility(ITaxonomy, name=theme_item)
    utility_district = sm.queryUtility(ITaxonomy, name=district_item)
    utility_iam = sm.queryUtility(ITaxonomy, name=iam_item)
    utility_locality = sm.queryUtility(ITaxonomy, name=locality_item)

    if utility_theme and utility_district and utility_iam and utility_locality:
        return

    create_taxonomy_object(data_theme, portal)
    create_taxonomy_object(data_district, portal)
    create_taxonomy_object(data_iam, portal)
    create_taxonomy_object(data_locality, portal)

    # remove taxonomy test
    item = "collective.taxonomy.test"
    utility = sm.queryUtility(ITaxonomy, name=item)
    if utility:
        utility.unregisterBehavior()
        sm.unregisterUtility(utility, ITaxonomy, name=item)
        sm.unregisterUtility(utility, IVocabularyFactory, name=item)
        sm.unregisterUtility(utility, ITranslationDomain, name=item)


def create_taxonomy_object(data_tax, portal):
    taxonomy = registerTaxonomy(
        api.portal.get(),
        name=data_tax["taxonomy"],
        title=data_tax["field_title"],
        description=data_tax["field_description"],
        default_language=data_tax["default_language"],
    )

    adapter = TaxonomyImportExportAdapter(portal)
    data_path = os.path.join(os.path.dirname(__file__), "data")
    file_path = os.path.join(data_path, data_tax["filename"])
    with open(file_path, "r") as f:
        adapter.importDocument(taxonomy, f.read().encode("utf8"))

    del data_tax["taxonomy"]
    del data_tax["filename"]
    taxonomy.registerBehavior(**data_tax)


def add_behavior(type_name, behavior_name):
    """Add a behavior to a type"""
    fti = queryUtility(IDexterityFTI, name=type_name)
    if not fti:
        return
    behaviors = list(fti.behaviors)
    if behavior_name not in behaviors:
        behaviors.append(behavior_name)
    fti._updateProperty("behaviors", tuple(behaviors))


def _activate_faceted_navigation(context, configuration=False, path=None):
    fpath = os.path.dirname(__file__) + path
    utils.activate_faceted_navigation(context, fpath)


def _publish(obj):
    if api.content.get_state(obj) != "published":
        api.content.transition(obj, transition="publish")


def uninstall(context):
    """Uninstall script"""
    # Do something at the end of the uninstallation of this package.



# -*- coding: utf-8 -*-
from ideabox.policy import _
from plone.supermodel.model import Schema
from plone.tiles import Tile
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope import schema


class ITimelineTile(Schema):
    """A tile that displays a listing of content items"""

    steps = schema.Text(
        title=_(u"Steps for timeline"),
        description=_(u"Title; Date; Link"),
        required=True,
    )


class TimelineTile(Tile):
    """A tile that displays a listing of content items"""

    template = ViewPageTemplateFile("templates/timeline.pt")

    def __call__(self):
        return self.template()

    def steps(self):
        steps = self.data["steps"]
        lines = steps.split("\r\n")
        data_steps = []
        for line in lines:
            data_line = line.split(";")
            if len(data_line) > 0 and len(data_line) < 4:
                step = {"title": "", "date": "", "link": ""}
                step["title"] = data_line[0]
                if len(data_line) > 1:
                    step["date"] = data_line[1]
                if len(data_line) > 2:
                    step["link"] = data_line[2]
                data_steps.append(step)
        return data_steps



# -*- coding: utf-8 -*-

from collective.taxonomy.interfaces import ITaxonomy
from ideabox.policy import _
from ideabox.policy.setuphandlers import create_taxonomy_object
from plone import api
from plone.registry import field
from plone.registry import Record
from plone.registry.interfaces import IRegistry
from zope.component import getUtility
from zope.i18n import translate
from zope.schema.interfaces import IVocabularyFactory

import logging

logger = logging.getLogger("ideabox.policy")


def reload_content_types(context):
    """Reload content types"""
    portal_setup = api.portal.get_tool("portal_setup")
    portal_setup.runImportStepFromProfile("profile-ideabox.policy:default", "typeinfo")


def fix_comments_1001(context):
    from plone.app.discussion.interfaces import IConversation

    brains = api.content.find(
        context=api.portal.get(),
        total_comments={"query": [1, 1000], "range": "min:max"},
    )
    for brain in brains:
        obj = brain.getObject()
        conversation = IConversation(obj, None)
        for thread in conversation.getThreads():
            comment = thread["comment"]
            if comment.author_name == comment.author_email:
                user = api.user.get(username=comment.author_username)
                infos = [user.getProperty("first_name"), user.getProperty("last_name")]
                comment.author_name = " ".join([i for i in infos if i])


def to_1002(context):
    """
    Fix project_district index
    """
    catalog = api.portal.get_tool("portal_catalog")
    catalog.delIndex("project_district")
    portal_setup = api.portal.get_tool("portal_setup")
    portal_setup.runImportStepFromProfile("profile-ideabox.policy:default", "catalog")

    for brain in api.content.find(portal_type="Project"):
        brain.getObject().reindexObject(idxs=["project_district"])


def to_1003(context):
    portal_setup = api.portal.get_tool("portal_setup")
    portal_setup.runImportStepFromProfile("profile-ideabox.policy:default", "viewlets")


def to_1005(context):
    """Add new tiles for priority action"""
    registry = getUtility(IRegistry)
    records = registry.records

    tiles = {
        "plone.app.mosaic.app_tiles.priorityactions": {
            "name": "ideabox.policy.priority_actions",
            "label": "Latest priority actions",
            "category": "advanced",
            "tile_type": "app",
            "default_value": "",
            "read_only": False,
            "settings": True,
            "favorite": False,
            "rich_text": False,
            "weight": 10,
        },
        "plone.app.mosaic.app_tiles.random_priorityactions": {
            "name": "ideabox.policy.random_priority_actions",
            "label": "Random priority actions",
            "category": "advanced",
            "tile_type": "app",
            "default_value": "",
            "read_only": False,
            "settings": True,
            "favorite": False,
            "rich_text": False,
            "weight": 10,
        },
    }
    types = {
        "name": (field.TextLine, "Name"),
        "label": (field.TextLine, "Label"),
        "category": (field.TextLine, "Category"),
        "tile_type": (field.TextLine, "Type"),
        "default_value": (field.TextLine, "Default value"),
        "read_only": (field.Bool, "Read only"),
        "settings": (field.Bool, "Settings"),
        "favorite": (field.Bool, "Favorite"),
        "rich_text": (field.Bool, "Rich Text"),
        "weight": (field.Int, "Weight"),
    }

    for name, values in tiles.items():
        for key, value in values.items():
            r_key = "{0}.{1}".format(name, key)
            field_type, title = types[key]
            required = True
            if key == "default_value":
                required = False
            if r_key in records:
                continue
            record = Record(field_type(title=title, required=required), value=value)
            records[r_key] = record


def to_1007(context):
    """Add new tiles for newsletter"""
    registry = getUtility(IRegistry)
    records = registry.records

    tiles = {
        "plone.app.mosaic.app_tiles.newsletter": {
            "name": "ideabox.policy.newsletter",
            "label": "Newsletter",
            "category": "advanced",
            "tile_type": "app",
            "default_value": "",
            "read_only": False,
            "settings": True,
            "favorite": False,
            "rich_text": False,
            "weight": 10,
        }
    }
    types = {
        "name": (field.TextLine, "Name"),
        "label": (field.TextLine, "Label"),
        "category": (field.TextLine, "Category"),
        "tile_type": (field.TextLine, "Type"),
        "default_value": (field.TextLine, "Default value"),
        "read_only": (field.Bool, "Read only"),
        "settings": (field.Bool, "Settings"),
        "favorite": (field.Bool, "Favorite"),
        "rich_text": (field.Bool, "Rich Text"),
        "weight": (field.Int, "Weight"),
    }

    for name, values in tiles.items():
        for key, value in values.items():
            r_key = "{0}.{1}".format(name, key)
            field_type, title = types[key]
            required = True
            if key == "default_value":
                required = False
            if r_key in records:
                continue
            record = Record(field_type(title=title, required=required), value=value)
            records[r_key] = record


def to_1008(context):
    """Add new taxonomy for locality (user profile)"""
    current_lang = api.portal.get_current_language()[:2]
    data_locality = {
        "taxonomy": "locality",
        "field_title": translate(
            _("Locality (Registration form)"), target_language=current_lang
        ),
        "field_description": "",
        "default_language": "fr",
        "filename": "taxonomy-settings-locality.xml",
    }

    portal = api.portal.get()
    sm = portal.getSiteManager()
    locality_item = "collective.taxonomy.locality"
    utility_locality = sm.queryUtility(ITaxonomy, name=locality_item)

    if not utility_locality:
        create_taxonomy_object(data_locality, portal)


def to_1009(context):
    """Add legal information text field in registry"""
    registry = getUtility(IRegistry)
    records = registry.records

    if (
        "ideabox.policy.browser.controlpanel.IIdeaBoxSettingsSchema.legal_information_text"
        in records
    ):  # noqa
        return

    logger.info(
        "Adding ideabox.policy.browser.controlpanel.IIdeaBoxSettingsSchema.legal_information_text to registry"
    )  # noqa
    record = Record(
        field.Text(
            title=_(u"Legal information text"),
            required=False,
            description=_(u"Legal information text"),
        )
    )
    records[
        "ideabox.policy.browser.controlpanel.IIdeaBoxSettingsSchema.legal_information_text"
    ] = record



# -*- coding: utf-8 -*-

from collective.z3cform.select2.widget.widget import MultiSelect2FieldWidget
from ideabox.policy import _
from ideabox.policy import logger
from ideabox.policy.content.project import IProject
from ideabox.policy.utils import execute_under_admin
from plone import api
from plone.namedfile.field import NamedBlobImage
from z3c.form import button
from z3c.form.field import Fields
from z3c.form.form import Form
from z3c.form.interfaces import HIDDEN_MODE
from z3c.form.interfaces import IFieldsForm
from zope import schema
from zope.i18n import translate
from zope.interface import implementer


class IProjectSubmission(IProject):
    title = schema.TextLine(title=_(u"Title"), required=True)

    project_image = NamedBlobImage(title=_(u"Project image"), required=False)


@implementer(IFieldsForm)
class ProjectSubmissionForm(Form):

    fields = Fields(IProjectSubmission).select(
        "title",
        "project_theme",
        "project_district",
        "body",
        "project_image",
        "original_author",
    )
    fields["project_theme"].widgetFactory = MultiSelect2FieldWidget
    fields["project_district"].widgetFactory = MultiSelect2FieldWidget

    fields["original_author"].mode = HIDDEN_MODE

    ignoreContext = True

    def update(self):
        super(ProjectSubmissionForm, self).update()
        self.widgets["original_author"].value = api.user.get_current().getProperty(
            "fullname"
        )

    def send_mail(self, url):
        lang = api.portal.get_current_language()[:2]
        email = api.portal.get_registry_record(
            "ideabox.policy.browser.controlpanel.IIdeaBoxSettingsSchema.project_manager_email",
            default=None,
        )
        if email is None:
            logger.warn("missing email for project submission notification")
            return

        list_mail = email.split(";")

        body = translate(
            _(
                u"email_body_project_submission",
                default=u"""A new project has been created you can access it at the following url:
              ${url}
              """,
                mapping={u"url": url},
            ),
            target_language=lang,
        )
        for mail in list_mail:
            api.portal.send_email(
                recipient=mail,
                subject=translate(_(u"New project submission"), target_language=lang),
                body=body,
            )

    def send_request(self, data):
        context = self.context
        folder = "/".join(context.getPhysicalPath())
        if not folder.startswith("/"):
            folder = "/{0}".format(folder)
        container = api.content.get(path=folder)
        project_obj = execute_under_admin(
            container,
            api.content.create,
            type="Project",
            title=data["title"],
            project_theme=data["project_theme"],
            project_district=data["project_district"],
            body=data["body"],
            container=container,
            original_author=api.user.get_current().id,
        )
        execute_under_admin(
            container, api.content.transition, obj=project_obj, transition="deposit"
        )
        if data["project_image"]:
            execute_under_admin(
                project_obj,
                api.content.create,
                type="Image",
                title=data["project_image"].filename,
                image=data["project_image"],
                container=project_obj,
            )
        self.request.response.redirect(project_obj.absolute_url())
        self.send_mail(project_obj.absolute_url())

    @button.buttonAndHandler(_(u"Send"), name="send")
    def handleApply(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return

        self.send_request(data)



# -*- coding: utf-8 -*-

from ideabox.policy import _
from plone.dexterity.content import Container
from plone.supermodel import model
from zope import schema
from zope.interface import implementer


class ICampaign(model.Schema):
    project_submission = schema.Bool(
        title=_(u"Enable / Disable project submission"), default=False
    )


@implementer(ICampaign)
class Campaign(Container):
    pass



# -*- coding: utf-8 -*-
from Products.statusmessages.interfaces import IStatusMessage
from cioppino.twothumbs.rate import hateIt
from cioppino.twothumbs.rate import loveIt
from cioppino.twothumbs.rate import setupAnnotations
from collective.z3cform.select2.widget.widget import MultiSelect2FieldWidget
from ideabox.policy import _
from ideabox.policy.userdataschema import IEnhancedUserDataSchema
from plone import api
from plone import schema
from plone.autoform import directives as form
from plone.i18n.normalizer.interfaces import IIDNormalizer
from z3c.form import button
from z3c.form.browser.radio import RadioFieldWidget
from z3c.form.field import Fields
from z3c.form.form import Form
from z3c.form.interfaces import IFieldsForm
from zope.component import getUtility
from zope.i18n import translate
from zope.interface import implementer

import hashlib
import password_generator
import time


class IVoteEncoding(IEnhancedUserDataSchema):
    mail = schema.Email(title=_(u"Email"), required=False)

    form.widget(vote=RadioFieldWidget)
    vote = schema.Choice(
        title=_(u"Vote"), required=True, vocabulary=u"ideabox.vocabularies.vote"
    )

    form.widget(project=MultiSelect2FieldWidget)
    project = schema.List(
        title=_(u"Project(s)"),
        value_type=schema.Choice(
            title=_(u"Project"), vocabulary=u"ideabox.vocabularies.projects"
        ),
        required=True,
    )


@implementer(IFieldsForm)
class VoteEncodingForm(Form):
    label = _(u"Vote encoding")
    fields = Fields(IVoteEncoding).select(
        "last_name",
        "first_name",
        "gender",
        "mail",
        "birthdate",
        "address",
        "zip_code",
        "iam",
        "vote",
        "project",
    )

    fields["project"].widgetFactory = MultiSelect2FieldWidget
    fields["vote"].widgetFactory = RadioFieldWidget
    fields["gender"].widgetFactory = RadioFieldWidget

    _required_fields = ("vote", "project")

    ignoreContext = True

    def update(self):
        for name, field in self.fields.items():
            field.field.required = name in self._required_fields
        super(VoteEncodingForm, self).update()

    def create_user(self, data):
        if data["mail"]:
            existing_user = api.user.get(userid=data["mail"].lower())
            if existing_user:
                self.update_user(existing_user, data)
                return existing_user
        properties = dict(
            last_name=data["last_name"]
            or translate(_(u"Anonymous"), context=self.request),
            first_name=data["first_name"],
            gender=data["gender"],
            birthdate=data["birthdate"],
            zip_code=data["zip_code"],
            iam=data["iam"],
            address=data["address"],
        )
        if not data["mail"]:
            normalizer = getUtility(IIDNormalizer)
            mail = "{0}_{1}@liege2025.be".format(
                normalizer.normalize(data["last_name"])[:15],
                hashlib.md5(str(time.time())).hexdigest()[:10],
            )
        else:
            mail = data["mail"]

        return api.user.create(
            email=mail.lower(),
            password="@1aA{0}".format(password_generator.generate()),
            properties=properties,
        )

    def update_user(self, user, data):
        updates = {}
        new_properties = {
            "last_name": data["last_name"]
            or translate(_(u"Anonymous"), context=self.request),
            "first_name": data["first_name"],
            "gender": data["gender"],
            "birthdate": data["birthdate"],
            "zip_code": data["zip_code"],
            "iam": data["iam"],
            "address": data["address"],
        }
        for field, new_value in new_properties.items():
            if not new_value:
                continue
            if user.getProperty(field) != new_value:
                updates[field] = new_value
        user.setMemberProperties(mapping=updates)

    def send_request(self, data):
        user = self.create_user(data)
        if data["vote"] == "FOR":
            for project in data["project"]:
                context = api.content.find(UID=project)[0].getObject()
                setupAnnotations(context)
                loveIt(context, userid=user.id)
        else:
            for project in data["project"]:
                context = api.content.find(UID=project)[0].getObject()
                setupAnnotations(context)
                hateIt(context, userid=user.id)

        self.request.response.redirect(
            "{0}/@@vote_encoding".format(self.context.absolute_url())
        )

    @button.buttonAndHandler(_(u"Send"), name="send")
    def handleApply(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return

        self.send_request(data)
        messages = IStatusMessage(self.request)
        messages.add(_(u"The votes have been encoded correctly"), type=u"info")



# -*- coding: utf-8 -*-

from eea.facetednavigation.browser.app.query import FacetedQueryHandler


class IdeaboxFacetedQueryHandler(FacetedQueryHandler):
    def query(self, *args, **kwargs):
        results = super(IdeaboxFacetedQueryHandler, self).query(*args, **kwargs)
        # Iterate over a batch breaks batch mecanisms
        # query = self.criteria()
        # if "sort_on" in query and query["sort_on"] == "random_sort":
        #     results = [e for e in results]
        #     shuffle(results)
        return results



# -*- coding: utf-8 -*-

from ideabox.policy import utils
from ideabox.policy.content.project import IProject
from plone.protect.auto import safeWrite
from tempfile import mkstemp
from zope.globalrequest import getRequest

import os


def project_image_changed(obj, event):
    if hasattr(obj, "aq_parent") is False:
        return
    if IProject.providedBy(obj.aq_parent):
        obj.aq_parent.reindexObject(idxs=["project_picture"])


def project_added(obj, event):
    request = getRequest()
    safeWrite(obj, request)
    obj.allow_discussion = True


def campaign_added(obj, event):
    fpath = os.path.join(
        os.path.dirname(__file__), "..", "faceted", "config", "campaign.xml"
    )
    xml = ""
    with open(fpath, "r") as config:
        xml = config.read()

    xml = xml.replace(
        '<property name="default">##PATH##</property>',
        '<property name="default">{0}</property>'.format(
            "/{0}".format("/".join(obj.absolute_url_path().split("/")[2:]))
        ),
    )
    fd, path = mkstemp()
    with open(path, "w") as f:
        f.write(xml)
    os.close(fd)
    utils.activate_faceted_navigation(obj, path)
    request = getRequest()
    request.response.redirect(obj.absolute_url())
    utils.disable_portlets(obj)
    utils.set_faceted_view(obj, "faceted-project")



# -*- coding: utf-8 -*-

from Products.Five import BrowserView
from ideabox.policy.utils import can_view_rating
from plone import api
from zope.component import getUtility
from zope.schema.interfaces import IVocabularyFactory


class ProjectsView(BrowserView):
    @property
    def default_image(self):
        return "{0}/project_default.jpg".format(api.portal.get().absolute_url())

    def rating(self, context):
        return can_view_rating(context)

    def get_theme(self, key):
        if not hasattr(self, "_themes"):
            factory = getUtility(IVocabularyFactory, "collective.taxonomy.theme")
            self._themes = factory(self.context)
        try:
            return self._themes.getTerm(key).title
        except KeyError:
            return ""

    def get_path(self):
        context = self.context
        return "/".join(context.getPhysicalPath())

    def can_submit_project(self):
        if self.context.portal_type != "campaign":
            return False
        return getattr(self.context, "project_submission", False)



# -*- coding: utf-8 -*-

from eea.facetednavigation.widgets import ViewPageTemplateFile
from eea.facetednavigation.widgets.widget import Widget as AbstractWidget
from ideabox.policy import _
from zope import schema
from z3c.form import field
from eea.facetednavigation.widgets.interfaces import ISchema
from eea.facetednavigation.widgets.interfaces import LayoutSchemata
from eea.facetednavigation.widgets.interfaces import DefaultSchemata as DS
from eea.facetednavigation import EEAMessageFactory as EEAMF_


class IProjectSortingSchema(ISchema):
    vocabulary = schema.Choice(
        title=EEAMF_(u"Filter from vocabulary"),
        description=EEAMF_(
            u"Vocabulary to use to filter sorting criteria. "
            u"Leave empty for default sorting criteria."
        ),
        vocabulary=u"eea.faceted.vocabularies.PortalVocabularies",
        required=True,
    )


class DefaultSchemata(DS):
    """Schemata default"""

    fields = field.Fields(IProjectSortingSchema).select(
        u"title", u"vocabulary", u"default"
    )


class Widget(AbstractWidget):
    """Widget"""

    widget_type = "project_sorting"
    widget_label = _("Project Sorting")

    groups = (DefaultSchemata, LayoutSchemata)
    index = ViewPageTemplateFile("project_sorting_widget.pt")

    @property
    def css_class(self):
        """Widget specific css class"""
        base = super(Widget, self).css_class
        return "faceted-sorting-widget {0}".format(base)

    @property
    def default(self):
        """Return default sorting values"""
        default = self.data.get("default", "")
        if not default:
            return ()
        reverse = False
        if "(reverse)" in default:
            default = default.replace("(reverse)", "", 1)
            reverse = True
        default = default.strip()
        return (default, reverse)

    def query(self, form):
        """Get value from form and return a catalog dict query"""
        query = {}

        if self.hidden:
            default = self.default
            sort_on = default[0] if default else None
            reverse = default[1] if len(default) > 1 else False
        else:
            sort_on = form.get(self.data.getId(), "")
            reverse = form.get("reversed", False)

        if sort_on:
            query["sort_on"] = sort_on

        if reverse:
            query["sort_order"] = "descending"
        else:
            query["sort_order"] = "ascending"

        return query

    def validateAddCriterion(self, indexId, criteriaType):
        """Is criteriaType acceptable criteria for indexId"""
        return True

    def vocabulary(self, **kwargs):
        """Return data vocabulary"""
        vocab = self.portal_vocabulary()
        vocab_fields = [(x[0].replace("term.", "", 1), x[1], "") for x in vocab]
        return vocab_fields



# -*- coding: utf-8 -*-

from collective.excelexport.exportables.dexterityfields import BaseFieldRenderer
from collective.excelexport.exportables.dexterityfields import CollectionFieldRenderer
from collective.excelexport.exportables.dexterityfields import (
    DexterityFieldsExportableFactory,
)
from collective.excelexport.exportables.dexterityfields import TextFieldRenderer
from ideabox.policy import _
from ideabox.policy.export.interfaces import IExtendedProjectExportable
from ideabox.policy.interfaces import IIdeaboxPolicyLayer
from plone import api
from plone.app.textfield.interfaces import IRichText
from Products.CMFPlone.utils import safe_unicode
from z3c.form.interfaces import NO_VALUE
from zope.annotation.interfaces import IAnnotations
from zope.component import adapts
from zope.component import getAdapters
from zope.component import getUtility
from zope.i18n import translate
from zope.interface import Interface
from zope.schema.interfaces import ICollection
from zope.schema.interfaces import IText
from zope.schema.interfaces import IVocabularyFactory


class FullTextFieldRenderer(TextFieldRenderer):
    adapts(IText, Interface, IIdeaboxPolicyLayer)

    def render_value(self, obj):
        """Gets the value to render in excel file from content value"""
        value = self.get_value(obj)
        if not value or value == NO_VALUE:
            return ""

        text = safe_unicode(self._get_text(value))

        return text


class FullRichTextFieldRenderer(FullTextFieldRenderer):
    adapts(IRichText, Interface, IIdeaboxPolicyLayer)

    def _get_text(self, value):
        ptransforms = api.portal.get_tool("portal_transforms")
        return ptransforms.convert("html_to_text", value.output).getData().strip()


class ProjectExportablesFactory(DexterityFieldsExportableFactory):
    portal_types = ["Project"]

    def get_exportables(self):
        project_extended = [
            ad[1] for ad in getAdapters((self.context,), IExtendedProjectExportable)
        ]
        return project_extended


class ExtendedRenderer(BaseFieldRenderer):
    adapts(Interface)
    name = ""

    def __init__(self, context):
        self.context = context

    def __repr__(self):
        return "<%s - %s>" % (self.__class__.__name__, self.name)

    def render_header(self):
        return self.name


class UserPropertyRenderer(ExtendedRenderer):
    prop = ""

    def render_value(self, obj):
        try:
            value = obj.getProperty(self.prop)
            if isinstance(value, str):
                value = value.decode("utf8")
            return value
        except ValueError:
            return


class ProjectCollectionFieldRenderer(CollectionFieldRenderer):
    adapts(ICollection, Interface, IIdeaboxPolicyLayer)
    separator = u","


class PositiveRatingRenderer(ExtendedRenderer):
    name = _(u"Positive rating ")

    def render_value(self, obj):
        annotations = IAnnotations(obj)
        vote = annotations.get("cioppino.twothumbs.yays")
        if vote:
            return len(annotations["cioppino.twothumbs.yays"])
        return 0


class NegativeRatingRenderer(ExtendedRenderer):
    name = _(u"Negative rating ")

    def render_value(self, obj):
        annotations = IAnnotations(obj)
        vote = annotations.get("cioppino.twothumbs.nays")
        if vote:
            return len(annotations["cioppino.twothumbs.nays"])
        return 0


class PositiveVotersListRenderer(ExtendedRenderer):
    name = _(u"Voters in favour")

    def render_value(self, obj):
        annotations = IAnnotations(obj)
        voters = []
        vote = annotations.get("cioppino.twothumbs.yays")
        if vote:
            for voter in annotations["cioppino.twothumbs.yays"]:
                voters.append(voter)
        return ",".join(voters)


class NegativeVotersListRenderer(ExtendedRenderer):
    name = _(u"Voters opposed")

    def render_value(self, obj):
        annotations = IAnnotations(obj)
        voters = []
        vote = annotations.get("cioppino.twothumbs.nays")
        if vote:
            for voter in annotations["cioppino.twothumbs.nays"]:
                voters.append(voter)
        return ",".join(voters)


class UserIdRenderer(UserPropertyRenderer):
    name = _(u"User ID")
    prop = "id"


class UserLastNameRenderer(UserPropertyRenderer):
    name = _(u"Last name")
    prop = "last_name"


class UserFirstNameRenderer(UserPropertyRenderer):
    name = _(u"First name")
    prop = "first_name"


class UserAddressRenderer(UserPropertyRenderer):
    name = _(u"Address")
    prop = "address"


class UserGenderRenderer(UserPropertyRenderer):
    name = _(u"Gender")
    prop = "gender"

    def render_value(self, obj):
        value = super(UserGenderRenderer, self).render_value(obj)
        if value:
            factory = getUtility(IVocabularyFactory, "ideabox.vocabularies.gender")
            vocabulary = factory(self.context)
            try:
                return translate(
                    vocabulary.getTerm(obj.getProperty("gender")).title,
                    target_language=api.portal.get_current_language(),
                )
            except KeyError:
                return


class UserBirthdateRenderer(UserPropertyRenderer):
    name = _(u"Birthdate")
    prop = "birthdate"

    def render_collection_entry(self, obj, value):
        return value.strftime("%d/%m/%Y")

    def render_style(self, obj, base_style):
        base_style.num_format_str = "dd/mm/yyyy"
        return base_style

    def render_value(self, obj):
        value = super(UserBirthdateRenderer, self).render_value(obj)
        if value:
            return self.render_collection_entry(obj, value)


class UserIamRenderer(UserPropertyRenderer):
    name = _(u"I am")
    prop = "iam"

    def render_value(self, obj):
        value = super(UserIamRenderer, self).render_value(obj)
        if value:
            factory = getUtility(IVocabularyFactory, "collective.taxonomy.iam")
            vocabulary = factory(self.context)
            try:
                return translate(
                    vocabulary.getTerm(obj.getProperty("iam")).title,
                    target_language=api.portal.get_current_language(),
                )
            except KeyError:
                return


class UserZipCodeRenderer(UserPropertyRenderer):
    name = _(u"Zip code")
    prop = "zip_code"


class UserVotesRenderer(ExtendedRenderer):
    name = _(u"Votes")

    def render_value(self, obj):
        projects = []
        userid = obj.getProperty("id")
        portal = api.portal.get()["projets"]
        for project in api.content.find(context=portal, portal_type="Project"):
            annotations = IAnnotations(project.getObject())
            vote = annotations.get("cioppino.twothumbs.yays")
            if vote:
                if userid in annotations["cioppino.twothumbs.yays"]:
                    projects.append(project.Title)
        return projects



# -*- coding: utf-8 -*-

from ideabox.policy.form.project_submission import ProjectSubmissionForm
from plone import api
from plone.z3cform.layout import FormWrapper
from ideabox.policy.browser.controlpanel import IIdeaBoxSettingsSchema


class ProjectSubmissionView(FormWrapper):
    form = ProjectSubmissionForm

    def enable_submission(self):
        context = self.context
        return context.project_submission

    def legal_information_text(self):
        text = api.portal.get_registry_record(
            name="legal_information_text",
            interface=IIdeaBoxSettingsSchema,
            default="",
        )
        return text



# -*- coding: utf-8 -*-

from DateTime.DateTime import DateTime
from ideabox.policy.userdataschema import IEnhancedUserDataSchema
from plone.app.users.browser.account import AccountPanelSchemaAdapter

import datetime


class EnhancedUserDataSchemaAdapter(AccountPanelSchemaAdapter):
    schema = IEnhancedUserDataSchema

    def get_birthdate(self):
        bd = self._getProperty("birthdate")
        return None if bd == DateTime("1900/01/01 00:00:00") else bd.asdatetime().date()

    def set_birthdate(self, value):
        return self._setProperty(
            "birthdate",
            DateTime(datetime.datetime(value.year, value.month, value.day, 0, 0)),
        )

    birthdate = property(get_birthdate, set_birthdate)



# -*- coding: utf-8 -*-

from plone.app.contenttypes.browser import folder


class FolderView(folder.FolderView):
    """Override of plone.app.contenttypes.browser.folder.FolderView"""

    def results(self, **kwargs):
        """Return a content listing based result set with contents of the
        folder.

        :param **kwargs: Any keyword argument, which can be used for catalog
                         queries.
        :type  **kwargs: keyword argument

        :returns: plone.app.contentlisting based result set.
        :rtype: ``plone.app.contentlisting.interfaces.IContentListing`` based
                sequence.
        """
        results = super(FolderView, self).results(**kwargs)
        results._basesequence = [e for e in results if e.exclude_from_nav is False]
        return results



# -*- coding: utf-8 -*-
from ideabox.policy import _
from plone.supermodel.model import Schema
from plone.tiles import Tile
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope import schema


class INewsletterTile(Schema):
    """A tile that displays a listing of content items"""

    title = schema.TextLine(title=_(u"Title"), required=True, default=_(u"Newsletter"))

    newsletter_link = schema.TextLine(title=_(u"Newsletter link"), required=True)


class NewsletterTile(Tile):
    template = ViewPageTemplateFile("templates/newsletter.pt")

    def __call__(self):
        return self.template()

    @property
    def title(self):
        return self.data.get("title") or _(u"Newsletter")

    @property
    def newsletter_link(self):
        return self.data.get("newsletter_link")



# -*- coding: utf-8 -*-
"""Installer for the ideabox.policy package."""

from setuptools import find_packages
from setuptools import setup


long_description = "\n\n".join(
    [
        open("README.rst").read(),
        open("CONTRIBUTORS.rst").read(),
        open("CHANGES.rst").read(),
    ]
)


setup(
    name="ideabox.policy",
    version="3.4.1",
    description="IdeaBox",
    long_description=long_description,
    # Get more from https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        "Environment :: Web Environment",
        "Framework :: Plone",
        "Framework :: Plone :: 5.2",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
    ],
    keywords="Python Plone",
    author="Martin Peeters",
    author_email="martin.peeters@affinitic.be",
    url="https://pypi.python.org/pypi/ideabox.policy",
    license="GPL version 2",
    packages=find_packages("src", exclude=["ez_setup"]),
    namespace_packages=["ideabox"],
    package_dir={"": "src"},
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        "Products.GenericSetup>=1.8.2",
        "Products.BeakerSessionDataManager",
        "cioppino.twothumbs",
        "collective.behavior.banner",
        "collective.behavior.richdescription",
        "collective.excelexport",
        "collective.disclaimer",
        "collective.taxonomy",
        "collective.z3cform.datetimewidget",
        "collective.z3cform.select2",
        "iaweb.mosaic",
        "ideabox.diazotheme.spirit",
        "ideabox.theme",
        "ideabox.stats",
        "ideabox.restapi",
        "imio.gdpr",
        "pas.plugins.imio",
        "password-generator",
        "plone.api",
        "plone.app.discussion",
        "plone.app.imagecropping",
        "plone.app.mosaic",
        "plone.dexterity",
        "setuptools",
        "z3c.jbot",
    ],
    extras_require={
        "test": [
            "plone.app.testing",
            "plone.testing",
            "plone.app.robotframework[debug]",
        ]
    },
    entry_points="""
    [z3c.autoinclude.plugin]
    target = plone
    """,
)



# -*- coding: utf-8 -*-

from AccessControl import getSecurityManager
from AccessControl.SecurityManagement import newSecurityManager
from AccessControl.SecurityManagement import setSecurityManager
from AccessControl.User import Super as BaseUnrestrictedUser
from datetime import datetime
from eea.facetednavigation.layout.layout import FacetedLayout
from ideabox.policy import _
from ideabox.policy import vocabularies
from plone import api
from plone.portlets.constants import CONTEXT_CATEGORY
from plone.portlets.interfaces import ILocalPortletAssignmentManager
from plone.portlets.interfaces import IPortletManager
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.i18n import translate


def token_type_recovery(value):
    value = value.decode("utf8")
    vocabulary = vocabularies.ThemeVocabulary(None)
    return [
        e.token for e in vocabulary.by_value.values() if translate(e.title) == value
    ][0]


class UnrestrictedUser(BaseUnrestrictedUser):
    """Unrestricted user that still has an id."""

    def getId(self):
        """Return the ID of the user."""
        return self.getUserName()


def execute_under_admin(portal, function, *args, **kwargs):
    """Execude code under admin privileges"""
    sm = getSecurityManager()
    try:
        try:
            tmp_user = UnrestrictedUser("admin", "", [""], "")
            # Wrap the user in the acquisition context of the portal
            tmp_user = tmp_user.__of__(portal.acl_users)
            newSecurityManager(None, tmp_user)
            # Call the function
            return function(*args, **kwargs)
        except:
            # If special exception handlers are needed, run them here
            raise
    finally:
        # Restore the old security manager
        setSecurityManager(sm)


def review_state(context):
    return api.content.get_state(obj=context)


def can_view_rating(context):
    _rating_states = ("vote", "result_analysis", "rejected")
    return review_state(context) in _rating_states


def now():
    return datetime.now()


def activate_faceted_navigation(context, config_path=""):
    subtyper = context.restrictedTraverse("@@faceted_subtyper")
    if subtyper.is_faceted:
        return
    subtyper.enable()
    with open(config_path, "rb") as config:
        context.unrestrictedTraverse("@@faceted_exportimport").import_xml(
            import_file=config
        )


def set_faceted_view(context, view_name):
    layout = FacetedLayout(context)
    layout.update_layout(layout=view_name)


def localized_month(value, request):
    keys = {
        "January": _("January"),
        "February": _("February"),
        "March": _("March"),
        "April": _("April"),
        "May": _("May"),
        "June": _("June"),
        "July": _("July"),
        "August": _("August"),
        "September": _("September"),
        "October": _("October"),
        "November": _("November"),
        "December": _("December"),
        "Jan": _("Jan"),
        "Feb": _("Feb"),
        "Mar": _("Mar"),
        "Apr": _("Apr"),
        "Jun": _("Jun"),
        "Jul": _("Jul"),
        "Aug": _("Aug"),
        "Sep": _("Sep"),
        "Oct": _("Oct"),
        "Nov": _("Nov"),
        "Dec": _("Dec"),
    }
    for k, v in keys.items():
        if k in value:
            value = value.replace(k, translate(v, context=request))
    return value


def disable_portlets(context, disabled=("plone.leftcolumn", "plone.rightcolumn")):
    for manager_name in disabled:
        manager = getUtility(IPortletManager, name=manager_name)
        blacklist = getMultiAdapter((context, manager), ILocalPortletAssignmentManager)
        blacklist.setBlacklistStatus(CONTEXT_CATEGORY, True)
